@@ -43,16 +43,14 @@ public:
         PACKAGE
     };
 
-    static constexpr auto iinfo = InputInfo ()
-        .with_exts (exts);
-
-    constexpr AdPlugXMMS () : InputPlugin (info, iinfo) {}
+    constexpr AdPlugXMMS () : InputPlugin (info, InputInfo ()
+        .with_exts (exts)) {}
 
     bool init ();
     void cleanup ();
 
     bool is_our_file (const char * filename, VFSFile & file);
-    Tuple read_tuple (const char * filename, VFSFile & file);
+    bool read_tag (const char * filename, VFSFile & file, Tuple & tuple, Index<char> * image);
     bool play (const char * filename, VFSFile & file);
 };
 
@@ -86,7 +84,6 @@ const char * const AdPlugXMMS::exts[] = {
 static struct {
   int freq = 44100l;
   bool bit16 = true, stereo = false, endless = false;
-  CPlayers players = CAdPlug::getPlayers();
 } conf;
 
 // Player variables
@@ -122,43 +119,33 @@ dbg_printf (const char *fmt, ...)
 
 #endif
 
-static CPlayer *
-factory (VFSFile & fd, Copl * newopl)
-{
-  return CAdPlug::factory (fd, newopl, conf.players);
-}
-
 /***** Main player (!! threaded !!) *****/
 
-Tuple AdPlugXMMS::read_tuple (const char * filename, VFSFile & fd)
+bool AdPlugXMMS::read_tag (const char * filename, VFSFile & file, Tuple & tuple,
+ Index<char> * image)
 {
-  Tuple tuple;
   CSilentopl tmpopl;
 
-  if (!fd)
-    return tuple;
+  CFileProvider fp (file);
+  CPlayer *p = CAdPlug::factory (filename, &tmpopl, fp);
 
-  CPlayer *p = factory (fd, &tmpopl);
+  if (! p)
+    return false;
 
-  if (p)
-  {
-    tuple.set_filename (filename);
+  if (! p->getauthor().empty())
+    tuple.set_str (Tuple::Artist, p->getauthor().c_str());
 
-    if (! p->getauthor().empty())
-      tuple.set_str (Tuple::Artist, p->getauthor().c_str());
+  if (! p->gettitle().empty())
+    tuple.set_str (Tuple::Title, p->gettitle().c_str());
+  else if (! p->getdesc().empty())
+    tuple.set_str (Tuple::Title, p->getdesc().c_str());
 
-    if (! p->gettitle().empty())
-      tuple.set_str (Tuple::Title, p->gettitle().c_str());
-    else if (! p->getdesc().empty())
-      tuple.set_str (Tuple::Title, p->getdesc().c_str());
+  tuple.set_str (Tuple::Codec, p->gettype().c_str());
+  tuple.set_str (Tuple::Quality, _("sequenced"));
+  tuple.set_int (Tuple::Length, p->songlength (plr.subsong));
+  delete p;
 
-    tuple.set_str (Tuple::Codec, p->gettype().c_str());
-    tuple.set_str (Tuple::Quality, _("sequenced"));
-    tuple.set_int (Tuple::Length, p->songlength (plr.subsong));
-    delete p;
-  }
-
-  return tuple;
+  return true;
 }
 
 /* Main playback thread. Takes the filename to play as argument. */
@@ -182,7 +169,8 @@ bool AdPlugXMMS::play (const char * filename, VFSFile & fd)
 
   // Try to load module
   dbg_printf ("factory, ");
-  if (!(plr.p = factory (fd, &opl)))
+  CFileProvider fp (fd);
+  if (!(plr.p = CAdPlug::factory (filename, &opl, fp)))
   {
     dbg_printf ("error!\n");
     // MessageBox("AdPlug :: Error", "File could not be opened!", "Ok");
@@ -271,7 +259,8 @@ bool AdPlugXMMS::is_our_file (const char * filename, VFSFile & fd)
 {
   CSilentopl tmpopl;
 
-  CPlayer *p = factory (fd, &tmpopl);
+  CFileProvider fp (fd);
+  CPlayer *p = CAdPlug::factory (filename, &tmpopl, fp);
 
   dbg_printf ("adplug_is_our_file(\"%s\"): returned ", filename);
 
@@ -306,21 +295,6 @@ bool AdPlugXMMS::init ()
   conf.freq = aud_get_int (CFG_VERSION, "Frequency");
   conf.endless = aud_get_bool (CFG_VERSION, "Endless");
 
-  // Read file type exclusion list
-  dbg_printf ("exclusion, ");
-  {
-    String cfgstr = aud_get_str (CFG_VERSION, "Exclude");
-
-    if (cfgstr[0])
-    {
-        StringBuf exclude = str_concat ({cfgstr, ":"});
-        str_replace_char (exclude, ':', 0);
-
-        for (char * p = exclude; * p; p += strlen (p) + 1)
-            conf.players.remove (conf.players.lookup_filetype (p));
-    }
-  }
-
   // Load database from disk and hand it to AdPlug
   dbg_printf ("database");
   {
@@ -358,18 +332,4 @@ void AdPlugXMMS::cleanup ()
   aud_set_bool (CFG_VERSION, "Stereo", conf.stereo);
   aud_set_int (CFG_VERSION, "Frequency", conf.freq);
   aud_set_bool (CFG_VERSION, "Endless", conf.endless);
-
-  dbg_printf ("exclude, ");
-  std::string exclude;
-  for (CPlayers::const_iterator i = CAdPlug::getPlayers().begin ();
-       i != CAdPlug::getPlayers().end (); i++)
-    if (find (conf.players.begin (), conf.players.end (), *i) ==
-        conf.players.end ())
-    {
-      if (!exclude.empty ())
-        exclude += ":";
-      exclude += (*i)->filetype;
-    }
-
-  aud_set_str (CFG_VERSION, "Exclude", exclude.c_str ());
 }
