@@ -32,9 +32,8 @@
 #ifndef H_DEBUG
 #define H_DEBUG
 
-#define ADL_DEBUG
+#include <libaudcore/runtime.h>
 
-void AdPlug_LogFile(const char *filename);
-void AdPlug_LogWrite(const char *fmt, ...);
+#define AdPlug_LogWrite(...) AUDDBG(__VA_ARGS__)
 
 #endif
