@@ -1,17 +1,17 @@
 /*
  * Adplug - Replayer for many OPL2/OPL3 audio file formats.
  * Copyright (C) 1999 - 2006 Simon Peter, <dn.tlp@gmx.net>, et al.
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -23,28 +23,27 @@
 #define H_ADPLUG_FILEPROVIDER
 
 #include <string>
-#include "binio_virtual.h"
+#include "binio.h"
+
+class VFSFile;
 
 class CFileProvider
 {
 public:
-  virtual ~CFileProvider()
-    {
-    }
+  CFileProvider(VFSFile &file) :
+    m_file(file) {}
 
-  virtual binistream *open(VFSFile &) const = 0;
-  virtual void close(binistream *) const = 0;
+  binistream *open(std::string filename) const;
+
+  static void close(binistream *f)
+    { delete f; }
 
   static bool extension(const std::string &filename,
-			const std::string &extension);
+                        const std::string &extension);
   static unsigned long filesize(binistream *f);
-};
 
-class CProvider_Filesystem: public CFileProvider
-{
-public:
-  virtual binistream *open(VFSFile &) const;
-  virtual void close(binistream *f) const;
+private:
+  VFSFile &m_file;
 };
 
 #endif
