@@ -138,20 +138,23 @@ bool CmodPlayer::update()
       tone_portamento(chan,channel[chan].portainfo);
     else
       vibrato(chan,channel[chan].vibinfo1,channel[chan].vibinfo2);
-    case 10: if(del % 4)        // SA2 volume slide
-      break;
+    case 10:
+      if(del % 4)        // SA2 volume slide
+        break;
       if(info1)
         vol_up(chan,info1);
       else
         vol_down(chan,info2);
       setvolume(chan);
       break;
-    case 14: if(info1 == 3)     // retrig note
-      if(!(del % (info2+1)))
-        playnote(chan);
-      break;
-    case 16: if(del % 4)        // AMD volume slide
+    case 14:
+      if(info1 == 3)     // retrig note
+        if(!(del % (info2+1)))
+          playnote(chan);
       break;
+    case 16:
+      if(del % 4)        // AMD volume slide
+        break;
       if(info1)
         vol_up_alt(chan,info1);
       else
@@ -193,8 +196,8 @@ bool CmodPlayer::update()
   if(!resolve_order()) return !songend;
   pattnr = order[ord];
 
-  if(!rw) AdPlug_LogWrite("\nCmodPlayer::update(): Pattern: %d, Order: %d\n", pattnr, ord);
-  AdPlug_LogWrite("CmodPlayer::update():%3d|", rw);
+  if(!rw) AdPlug_LogWrite("\nCmodPlayer::update(): Pattern: %d, Order: %lu\n", pattnr, ord);
+  AdPlug_LogWrite("CmodPlayer::update():%3lu|", rw);
 
   // play row
   pattern_delay = 0;
@@ -202,7 +205,7 @@ bool CmodPlayer::update()
   for(chan = 0; chan < nchans; chan++) {
     oplchan = set_opl_chip(chan);
 
-    if(!(activechan >> (31 - chan)) & 1) {      // channel active?
+    if(!((activechan >> (31 - chan)) & 1)) {      // channel active?
       AdPlug_LogWrite("N/A|");
       continue;
     }
