@@ -45,7 +45,8 @@ const char * const pw_col_names[PW_COLS] = {
     N_("File path"),
     N_("File name"),
     N_("Custom title"),
-    N_("Bitrate")
+    N_("Bitrate"),
+    N_("Comment")
 };
 
 int pw_num_cols;
@@ -66,7 +67,8 @@ static const char * const pw_col_keys[PW_COLS] = {
     "path",
     "filename",
     "custom",
-    "bitrate"
+    "bitrate",
+    "comment"
 };
 
 static const int pw_default_widths[PW_COLS] = {
@@ -83,7 +85,8 @@ static const int pw_default_widths[PW_COLS] = {
     275,  // path
     275,  // filename
     275,  // custom title
-    10    // bitrate
+    10,   // bitrate
+    275   // comment
 };
 
 void pw_col_init ()
@@ -111,14 +114,13 @@ void pw_col_init ()
         pw_cols[pw_num_cols ++] = i;
     }
 
-    String widths = aud_get_str ("gtkui", "column_widths");
+    auto widths = str_list_to_index (aud_get_str ("gtkui", "column_widths"), ", ");
+    int nwidths = aud::min (widths.len (), (int) PW_COLS);
 
-    if (! str_to_int_array (widths, pw_col_widths, PW_COLS))
-    {
-        int dpi = audgui_get_dpi ();
-        for (int i = 0; i < PW_COLS; i ++)
-            pw_col_widths[i] = pw_default_widths[i] * dpi / 96;
-    }
+    for (int i = 0; i < nwidths; i ++)
+        pw_col_widths[i] = audgui_to_native_dpi (str_to_int (widths[i]));
+    for (int i = nwidths; i < PW_COLS; i ++)
+        pw_col_widths[i] = audgui_to_native_dpi (pw_default_widths[i]);
 }
 
 struct Column {
@@ -134,12 +136,12 @@ static void apply_changes ()
     int cols = chosen.len ();
     g_return_if_fail (cols <= PW_COLS);
 
-    ui_playlist_notebook_empty ();
+    pl_notebook_purge ();
 
     for (pw_num_cols = 0; pw_num_cols < cols; pw_num_cols ++)
         pw_cols[pw_num_cols] = chosen[pw_num_cols].column;
 
-    ui_playlist_notebook_populate ();
+    pl_notebook_populate ();
 }
 
 static void get_value (void * user, int row, int column, GValue * value)
@@ -350,6 +352,10 @@ void pw_col_save ()
     for (int i = 0; i < pw_num_cols; i ++)
         index.append (String (pw_col_keys[pw_cols[i]]));
 
+    int widths[PW_COLS];
+    for (int i = 0; i < PW_COLS; i ++)
+        widths[i] = audgui_to_portable_dpi (pw_col_widths[i]);
+
     aud_set_str ("gtkui", "playlist_columns", index_to_str_list (index, " "));
-    aud_set_str ("gtkui", "column_widths", int_array_to_str (pw_col_widths, PW_COLS));
+    aud_set_str ("gtkui", "column_widths", int_array_to_str (widths, PW_COLS));
 }
