@@ -33,7 +33,7 @@
 #include <libxml/HTMLparser.h>
 #include <libxml/xpath.h>
 
-#define AUD_PLUGIN_QT_ONLY
+#define AUD_GLIB_INTEGRATION
 #include <libaudcore/drct.h>
 #include <libaudcore/i18n.h>
 #include <libaudcore/plugin.h>
@@ -67,7 +67,10 @@ class LyricWikiQt : public GeneralPlugin
 public:
     static constexpr PluginInfo info = {
         N_("LyricWiki Plugin"),
-        PACKAGE
+        PACKAGE,
+        nullptr, // about
+        nullptr, // prefs
+        PluginQtOnly
     };
 
     constexpr LyricWikiQt () : GeneralPlugin (info, false) {}
@@ -84,11 +87,10 @@ static void libxml_error_handler (void * ctx, const char * msg, ...)
 {
 }
 
-/* g_free() returned text */
-static char * scrape_lyrics_from_lyricwiki_edit_page (const char * buf, int64_t len)
+static CharPtr scrape_lyrics_from_lyricwiki_edit_page (const char * buf, int64_t len)
 {
     xmlDocPtr doc;
-    char * ret = nullptr;
+    CharPtr ret;
 
     /*
      * temporarily set our error-handling functor to our suppression function,
@@ -143,12 +145,9 @@ give_up:
                  (GRegexMatchFlags) 0, nullptr);
                 g_regex_match (reg, (char *) lyric, G_REGEX_MATCH_NEWLINE_ANY, & match_info);
 
-                ret = g_match_info_fetch (match_info, 2);
-                if (! g_utf8_collate (ret, "<!-- PUT LYRICS HERE (and delete this entire line) -->"))
-                {
-                    g_free (ret);
-                    ret = g_strdup (_("No lyrics available"));
-                }
+                ret.capture (g_match_info_fetch (match_info, 2));
+                if (! strcmp_nocase (ret, "<!-- PUT LYRICS HERE (and delete this entire line) -->"))
+                    ret.capture (g_strdup (_("No lyrics available")));
 
                 g_regex_unref (reg);
             }
@@ -176,7 +175,7 @@ static String scrape_uri_from_lyricwiki_search_result (const char * buf, int64_t
     reg = g_regex_new ("<(lyrics?)>.*</\\1>", (GRegexCompileFlags)
      (G_REGEX_MULTILINE | G_REGEX_DOTALL | G_REGEX_UNGREEDY),
      (GRegexMatchFlags) 0, nullptr);
-    char * newbuf = g_regex_replace_literal (reg, buf, len, 0, "", G_REGEX_MATCH_NEWLINE_ANY, nullptr);
+    CharPtr newbuf (g_regex_replace_literal (reg, buf, len, 0, "", G_REGEX_MATCH_NEWLINE_ANY, nullptr));
     g_regex_unref (reg);
 
     /*
@@ -202,25 +201,60 @@ static String scrape_uri_from_lyricwiki_search_result (const char * buf, int64_t
         {
             if (xmlStrEqual(cur->name, (xmlChar *) "url"))
             {
-                xmlChar * lyric;
-                char * basename;
+                auto lyric = (char *) xmlNodeGetContent (cur);
 
-                lyric = xmlNodeGetContent (cur);
-                basename = g_path_get_basename ((char *) lyric);
+                // If the lyrics are unknown, LyricWiki returns a broken link
+                // to the edit page.  Extract the song ID (artist:title) from
+                // the URI and recreate a working link.
+                char * title = strstr (lyric, "title=");
+                if (title)
+                {
+                    title += 6;
 
-                uri = String (str_printf ("http://lyrics.wikia.com/index.php?"
-                 "action=edit&title=%s", basename));
+                    // Strip trailing "&action=edit"
+                    char * amp = strchr (title, '&');
+                    if (amp)
+                        * amp = 0;
 
-                g_free (basename);
-                xmlFree (lyric);
+                    // Spaces get replaced with plus signs for some reason.
+                    str_replace_char (title, '+', ' ');
+
+                    // LyricWiki interprets UTF-8 as ISO-8859-1, then "converts"
+                    // it to UTF-8 again.  Worse, it will sometimes corrupt only
+                    // the song title in this way while leaving the artist name
+                    // intact.  So we have to percent-decode the URI, split the
+                    // two strings apart, repair them separately, and then
+                    // rebuild the URI.
+                    auto strings = str_list_to_index (str_decode_percent (title), ":");
+                    for (String & s : strings)
+                    {
+                        StringBuf orig_utf8 = str_convert (s, -1, "UTF-8", "ISO-8859-1");
+                        if (orig_utf8 && g_utf8_validate (orig_utf8, -1, nullptr))
+                            s = String (orig_utf8);
+                    }
+
+                    uri = String (str_printf ("http://lyrics.wikia.com/index.php?"
+                     "action=edit&title=%s", (const char *) str_encode_percent
+                     (index_to_str_list (strings, ":"))));
+                }
+                else
+                {
+                    // Convert normal lyrics link to edit page link
+                    char * slash = strrchr (lyric, '/');
+                    if (slash && ! strstr (slash, "lyrics.wikia.com"))
+                        uri = String (str_printf ("http://lyrics.wikia.com/index.php?"
+                         "action=edit&title=%s", slash + 1));
+                    else
+                        uri = String ("N/A");
+                }
+
+                xmlFree ((xmlChar *) lyric);
             }
         }
 
         xmlFreeDoc (doc);
     }
 
-    g_free (newbuf);
-
     return uri;
 }
 
@@ -238,7 +272,7 @@ static void get_lyrics_step_3 (const char * uri, const Index<char> & buf, void *
         return;
     }
 
-    char * lyrics = scrape_lyrics_from_lyricwiki_edit_page (buf.begin (), buf.len ());
+    CharPtr lyrics = scrape_lyrics_from_lyricwiki_edit_page (buf.begin (), buf.len ());
 
     if (! lyrics)
     {
@@ -248,8 +282,6 @@ static void get_lyrics_step_3 (const char * uri, const Index<char> & buf, void *
     }
 
     update_lyrics_window (state.title, state.artist, lyrics);
-
-    g_free (lyrics);
 }
 
 static void get_lyrics_step_2 (const char * uri1, const Index<char> & buf, void *)
@@ -272,6 +304,12 @@ static void get_lyrics_step_2 (const char * uri1, const Index<char> & buf, void
          str_printf (_("Unable to parse %s"), uri1));
         return;
     }
+    else if (uri == String ("N/A"))
+    {
+        update_lyrics_window (state.title, state.artist,
+         _("No lyrics available"));
+        return;
+    }
 
     state.uri = uri;
 
