@@ -60,15 +60,13 @@ public:
         & prefs
     };
 
-    static constexpr auto iinfo = InputInfo (FlagSubtunes)
-        .with_exts (exts);
-
-    constexpr ModplugXMMS () : InputPlugin (info, iinfo) {}
+    constexpr ModplugXMMS () : InputPlugin (info, InputInfo (FlagSubtunes)
+        .with_exts (exts)) {}
 
     bool init ();
 
     bool is_our_file (const char * filename, VFSFile & file);
-    Tuple read_tuple (const char * filename, VFSFile & file);
+    bool read_tag (const char * filename, VFSFile & file, Tuple & tuple, Index<char> * image);
     bool play (const char * filename, VFSFile & file);
 
 private:
