@@ -1,4 +1,5 @@
-#include <libaudcore/plugin.h>
+#include <stdint.h>
+#include <libaudcore/tuple.h>
 
 int32_t psf2_start(uint8_t *, uint32_t length);
 int32_t psf2_execute(void (*update)(const void *, int));
