@@ -193,7 +193,7 @@ static u32 seektime;
 int psf_seek(u32 t)
 {
  seektime=t*441/10;
- if(seektime>sampcount) return(1);
+ if(seektime>=sampcount) return(1);
  return(0);
 }
 
@@ -493,8 +493,10 @@ int SPUasync(u32 cycles, void (*update)(const void *, int))
   // if(sr>32767 || sr < -32767) printf("Right: %d, %f\n",sl,asl);
   //}
 
-  if(sl>32767) sl=32767; if(sl<-32767) sl=-32767;
-  if(sr>32767) sr=32767; if(sr<-32767) sr=-32767;
+  if(sl>32767) sl=32767;
+  if(sl<-32767) sl=-32767;
+  if(sr>32767) sr=32767;
+  if(sr<-32767) sr=-32767;
 
   *pS++=sl;
   *pS++=sr;
@@ -560,6 +562,7 @@ int SPUinit(void)
  memset(spuMem,0,sizeof(spuMem));
  InitADSR();
  sampcount=ttemp=0;
+ seektime=0;
  #ifdef TIMEO
  begintime=gettime64();
  #endif
