@@ -20,15 +20,52 @@
 #include "playlist.h"
 #include "playlist_tabs.h"
 #include "menus.h"
+#include "search_bar.h"
 
+#include <QBoxLayout>
 #include <QKeyEvent>
 #include <QLineEdit>
 
+#include <libaudcore/i18n.h>
 #include <libaudcore/playlist.h>
 #include <libaudcore/runtime.h>
 
 #include <libaudqt/libaudqt.h>
 
+class LayoutWidget : public QWidget
+{
+public:
+    LayoutWidget (QWidget * parent, Playlist playlist, QMenu * contextMenu);
+
+    PlaylistWidget * playlistWidget () const
+        { return m_playlistWidget; }
+
+    void activateSearch ()
+    {
+        m_searchBar->show ();
+        m_searchBar->setFocus ();
+    }
+
+private:
+    PlaylistWidget * m_playlistWidget;
+    SearchBar * m_searchBar;
+};
+
+LayoutWidget::LayoutWidget (QWidget * parent, Playlist playlist, QMenu * contextMenu) :
+    QWidget (parent),
+    m_playlistWidget (new PlaylistWidget (this, playlist)),
+    m_searchBar (new SearchBar (this, m_playlistWidget))
+{
+    auto layout = audqt::make_vbox (this, 0);
+    layout->addWidget (m_playlistWidget);
+    layout->addWidget (m_searchBar);
+
+    m_playlistWidget->setContextMenu (contextMenu);
+    m_searchBar->hide ();
+}
+
+/* --------------------------------- */
+
 PlaylistTabs::PlaylistTabs (QWidget * parent) :
     QTabWidget (parent),
     m_pl_menu (qtui_build_pl_menu (this)),
@@ -43,49 +80,57 @@ PlaylistTabs::PlaylistTabs (QWidget * parent) :
 
     addRemovePlaylists ();
     updateTitles ();
-    setCurrentIndex (aud_playlist_get_active ());
+    setCurrentIndex (Playlist::active_playlist ().index ());
 
     connect (this, & QTabWidget::currentChanged, this, & PlaylistTabs::currentChangedTrigger);
 }
 
-PlaylistWidget * PlaylistTabs::createWidget (int list)
+PlaylistWidget * PlaylistTabs::currentPlaylistWidget () const
 {
-    int id = aud_playlist_get_unique_id (list);
-    auto widget = new PlaylistWidget (this, id);
-    widget->setContextMenu (m_pl_menu);
-    return widget;
+    return ((LayoutWidget *) currentWidget ())->playlistWidget ();
+}
+
+PlaylistWidget * PlaylistTabs::playlistWidget (int idx) const
+{
+    auto w = (LayoutWidget *) widget (idx);
+    return w ? w->playlistWidget () : nullptr;
+}
+
+void PlaylistTabs::activateSearch ()
+{
+    ((LayoutWidget *) currentWidget ())->activateSearch ();
 }
 
 void PlaylistTabs::addRemovePlaylists ()
 {
     int tabs = count ();
-    int playlists = aud_playlist_count ();
+    int playlists = Playlist::n_playlists ();
 
     for (int i = 0; i < tabs; i ++)
     {
-        auto widget = playlistWidget (i);
-        int playlist = widget->playlist ();
+        auto w = (LayoutWidget *) widget (i);
+        int list_idx = w->playlistWidget ()->playlist ().index ();
 
-        if (playlist < 0)
+        if (list_idx < 0)
         {
             removeTab (i);
-            delete widget;
+            delete w;
             tabs --;
             i --;
         }
-        else if (playlist != i)
+        else if (list_idx != i)
         {
             bool found = false;
 
             for (int j = i + 1; j < tabs; j ++)
             {
-                widget = playlistWidget (j);
-                playlist = widget->playlist ();
+                w = (LayoutWidget *) widget (j);
+                list_idx = w->playlistWidget ()->playlist ().index ();
 
-                if (playlist == i)
+                if (list_idx == i)
                 {
                     removeTab (j);
-                    insertTab (i, widget, QString ());
+                    insertTab (i, w, QString ());
                     found = true;
                     break;
                 }
@@ -93,7 +138,7 @@ void PlaylistTabs::addRemovePlaylists ()
 
             if (! found)
             {
-                insertTab (i, createWidget (i), QString ());
+                insertTab (i, new LayoutWidget (this, Playlist::by_index (i), m_pl_menu), QString ());
                 tabs ++;
             }
         }
@@ -101,7 +146,7 @@ void PlaylistTabs::addRemovePlaylists ()
 
     while (tabs < playlists)
     {
-        addTab (createWidget (tabs), QString ());
+        addTab (new LayoutWidget (this, Playlist::by_index (tabs), m_pl_menu), QString ());
         tabs ++;
     }
 }
@@ -110,18 +155,12 @@ void PlaylistTabs::updateTitles ()
 {
     int tabs = count ();
     for (int i = 0; i < tabs; i ++)
-        setTabTitle (i, aud_playlist_get_title (i));
-}
-
-void PlaylistTabs::filterTrigger (const QString & text)
-{
-    ((PlaylistWidget *) currentWidget ())->setFilter (text);
+        updateTabText (i);
 }
 
 void PlaylistTabs::currentChangedTrigger (int idx)
 {
-    cancelRename ();
-    aud_playlist_set_active (idx);
+    Playlist::by_index (idx).activate ();
 }
 
 QLineEdit * PlaylistTabs::getTabEdit (int idx)
@@ -129,22 +168,28 @@ QLineEdit * PlaylistTabs::getTabEdit (int idx)
     return dynamic_cast<QLineEdit *> (m_tabbar->tabButton (idx, QTabBar::LeftSide));
 }
 
-void PlaylistTabs::setTabTitle (int idx, const char * text)
+void PlaylistTabs::updateTabText (int idx)
 {
-    // escape ampersands for setTabText ()
-    auto title = QString (text).replace ("&", "&&");
+    QString title;
 
-    if (aud_get_bool ("qtui", "entry_count_visible"))
-        title += QString (" (%1)").arg (aud_playlist_entry_count (idx));
+    if (! getTabEdit (idx))
+    {
+        auto playlist = Playlist::by_index (idx);
+
+        // escape ampersands for setTabText ()
+        title = QString (playlist.get_title ()).replace ("&", "&&");
+
+        if (aud_get_bool ("qtui", "entry_count_visible"))
+            title += QString (" (%1)").arg (playlist.n_entries ());
+    }
 
     setTabText (idx, title);
 }
 
-void PlaylistTabs::setupTab (int idx, QWidget * button, const char * text, QWidget * * oldp)
+void PlaylistTabs::setupTab (int idx, QWidget * button, QWidget * * oldp)
 {
     QWidget * old = m_tabbar->tabButton (idx, QTabBar::LeftSide);
     m_tabbar->setTabButton (idx, QTabBar::LeftSide, button);
-    setTabTitle (idx, text);
 
     if (oldp)
         * oldp = old;
@@ -153,37 +198,25 @@ void PlaylistTabs::setupTab (int idx, QWidget * button, const char * text, QWidg
         old->setParent (nullptr);
         old->deleteLater ();
     }
+
+    updateTabText (idx);
 }
 
-void PlaylistTabs::tabEditedTrigger ()
-{
-    int idx = currentIndex ();
-    if (idx < 0)
-        return;
-
-    QLineEdit * edit = getTabEdit (idx);
-    if (! edit)
-        return;
-
-    QByteArray title = edit->text ().toUtf8 ();
-    aud_playlist_set_title (idx, title);
-
-    setupTab (idx, m_leftbtn, title, nullptr);
-    m_leftbtn = nullptr;
-}
-
-void PlaylistTabs::editTab (int idx)
+void PlaylistTabs::editTab (int idx, Playlist playlist)
 {
     QLineEdit * edit = getTabEdit (idx);
 
     if (! edit)
     {
-        edit = new QLineEdit ((const char *) aud_playlist_get_title (idx));
+        edit = new QLineEdit ((const char *) playlist.get_title ());
 
-        connect (edit, & QLineEdit::returnPressed, this, & PlaylistTabs::tabEditedTrigger);
+        connect (edit, & QLineEdit::returnPressed, [this, playlist, edit] ()
+        {
+            playlist.set_title (edit->text ().toUtf8 ());
+            cancelRename ();
+        });
 
-        setupTab (idx, edit, nullptr, & m_leftbtn);
-        setTabText (idx, nullptr);
+        setupTab (idx, edit, & m_leftbtn);
     }
 
     edit->selectAll ();
@@ -197,10 +230,7 @@ bool PlaylistTabs::eventFilter (QObject * obj, QEvent * e)
         QKeyEvent * ke = (QKeyEvent *) e;
 
         if (ke->key () == Qt::Key_Escape)
-        {
-            cancelRename ();
-            return true;
-        }
+            return cancelRename ();
     }
 
     return QTabWidget::eventFilter(obj, e);
@@ -209,30 +239,36 @@ bool PlaylistTabs::eventFilter (QObject * obj, QEvent * e)
 void PlaylistTabs::renameCurrent ()
 {
     int idx = currentIndex ();
+    auto playlist = currentPlaylistWidget ()->playlist ();
 
     if (! m_tabbar->isVisible ())
-        audqt::playlist_show_rename (idx);
+        audqt::playlist_show_rename (playlist);
     else
-        editTab (idx);
+        editTab (idx, playlist);
 }
 
-void PlaylistTabs::cancelRename ()
+bool PlaylistTabs::cancelRename ()
 {
+    bool cancelled = false;
+
     for (int i = 0; i < count (); i ++)
     {
         QLineEdit * edit = getTabEdit (i);
         if (! edit)
             continue;
 
-        setupTab (i, m_leftbtn, (const char *) aud_playlist_get_title (i), nullptr);
+        setupTab (i, m_leftbtn, nullptr);
         m_leftbtn = nullptr;
+        cancelled = true;
     }
+
+    return cancelled;
 }
 
 void PlaylistTabs::playlist_activate_cb ()
 {
-    if (! aud_playlist_update_pending ())
-        setCurrentIndex (aud_playlist_get_active ());
+    setCurrentIndex (Playlist::active_playlist ().index ());
+    cancelRename ();
 }
 
 void PlaylistTabs::playlist_update_cb (Playlist::UpdateLevel global_level)
@@ -242,15 +278,10 @@ void PlaylistTabs::playlist_update_cb (Playlist::UpdateLevel global_level)
     if (global_level >= Playlist::Metadata)
         updateTitles ();
 
-    int lists = aud_playlist_count ();
-    for (int list = 0; list < lists; list ++)
-    {
-        auto update = aud_playlist_update_detail (list);
-        if (update.level)
-            playlistWidget (list)->update (update);
-    }
+    for (int i = 0; i < count (); i ++)
+        playlistWidget (i)->playlistUpdate ();
 
-    setCurrentIndex (aud_playlist_get_active ());
+    setCurrentIndex (Playlist::active_playlist ().index ());
 }
 
 void PlaylistTabs::playlist_position_cb (int list)
@@ -267,12 +298,14 @@ PlaylistTabBar::PlaylistTabBar (QWidget * parent) : QTabBar (parent)
     updateSettings ();
 
     connect (this, & QTabBar::tabMoved, this, & PlaylistTabBar::tabMoved);
-    connect (this, & QTabBar::tabCloseRequested, this, & PlaylistTabBar::handleCloseRequest);
+    connect (this, & QTabBar::tabCloseRequested, [] (int idx) {
+        audqt::playlist_confirm_delete (Playlist::by_index (idx));
+    });
 }
 
 void PlaylistTabBar::tabMoved (int from, int to)
 {
-    aud_playlist_reorder (from, to, 1);
+    Playlist::reorder_playlists (from, to, 1);
 }
 
 void PlaylistTabBar::mousePressEvent (QMouseEvent * e)
@@ -280,8 +313,11 @@ void PlaylistTabBar::mousePressEvent (QMouseEvent * e)
     if (e->button () == Qt::MidButton)
     {
         int index = tabAt (e->pos ());
-        handleCloseRequest (index);
-        e->accept ();
+        if (index >= 0)
+        {
+            audqt::playlist_confirm_delete (Playlist::by_index (index));
+            e->accept ();
+        }
     }
 
     QTabBar::mousePressEvent (e);
@@ -293,21 +329,7 @@ void PlaylistTabBar::mouseDoubleClickEvent (QMouseEvent * e)
     if (idx < 0 || e->button () != Qt::LeftButton)
         return;
 
-    PlaylistTabs * p = (PlaylistTabs *) parent ();
-    PlaylistWidget * pl = p->playlistWidget (idx);
-
-    aud_playlist_play (pl->playlist ());
-}
-
-void PlaylistTabBar::handleCloseRequest (int idx)
-{
-    PlaylistTabs * p = (PlaylistTabs *) parent ();
-    PlaylistWidget * pl = p->playlistWidget (idx);
-
-    if (! pl)
-        return;
-
-    audqt::playlist_confirm_delete (pl->playlist ());
+    Playlist::by_index (idx).start_playback ();
 }
 
 void PlaylistTabBar::updateSettings ()
