@@ -27,8 +27,10 @@
 #include <QLabel>
 #include <QLineEdit>
 #include <QMenu>
+#include <QMimeData>
 #include <QPushButton>
 #include <QTreeView>
+#include <QUrl>
 
 #include <libaudcore/audstrings.h>
 #include <libaudcore/hook.h>
@@ -58,6 +60,7 @@ public:
     constexpr SearchToolQt () : GeneralPlugin (info, false) {}
 
     void * get_qt_widget ();
+    int take_message (const char * code, const void *, int);
 };
 
 EXPORT SearchToolQt aud_plugin_instance;
@@ -99,11 +102,6 @@ struct Item
     Item & operator= (Item &&) = default;
 };
 
-struct SearchState {
-    Index<const Item *> items;
-    int mask;
-};
-
 class ResultsModel : public QAbstractListModel
 {
 public:
@@ -112,9 +110,23 @@ public:
 protected:
     int rowCount (const QModelIndex & parent) const { return m_rows; }
     int columnCount (const QModelIndex & parent) const { return 1; }
-
     QVariant data (const QModelIndex & index, int role) const;
 
+    Qt::ItemFlags flags (const QModelIndex & index) const
+    {
+        if (index.isValid ())
+            return Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsEnabled;
+        else
+            return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
+    }
+
+    QStringList mimeTypes () const
+    {
+        return QStringList ("text/uri-list");
+    }
+
+    QMimeData * mimeData (const QModelIndexList & indexes) const;
+
 private:
     int m_rows = 0;
 };
@@ -127,33 +139,34 @@ protected:
 
 static StringBuf create_item_label (int row);
 
-static int playlist_id;
-static Index<String> search_terms;
+static Playlist s_playlist;
+static Index<String> s_search_terms;
 
 /* Note: added_table is accessed by multiple threads.
  * When adding = true, it may only be accessed by the playlist add thread.
  * When adding = false, it may only be accessed by the UI thread.
  * adding may only be set by the UI thread while holding adding_lock. */
-static TinyLock adding_lock;
-static bool adding = false;
-static SimpleHash<String, bool> added_table;
+static TinyLock s_adding_lock;
+static bool s_adding = false;
+static SimpleHash<String, bool> s_added_table;
 
-static SimpleHash<Key, Item> database;
-static bool database_valid;
-static Index<const Item *> items;
-static int hidden_items;
+static SimpleHash<Key, Item> s_database;
+static bool s_database_valid;
+static Index<const Item *> s_items;
+static int s_hidden_items;
 
-static QueuedFunc search_timer;
-static bool search_pending;
+static QueuedFunc s_search_timer;
+static bool s_search_pending;
 
-static ResultsModel model;
-static QLabel * help_label, * wait_label, * stats_label;
-static QTreeView * results_list;
-static QMenu * menu;
+static ResultsModel s_model;
+static QLabel * s_help_label, * s_wait_label, * s_stats_label;
+static QLineEdit * s_search_entry;
+static QTreeView * s_results_list;
+static QMenu * s_menu;
 
 void ResultsModel::update ()
 {
-    int rows = items.len ();
+    int rows = s_items.len ();
     int keep = aud::min (rows, m_rows);
 
     if (rows < m_rows)
@@ -187,44 +200,40 @@ QVariant ResultsModel::data (const QModelIndex & index, int role) const
 
 static void find_playlist ()
 {
-    playlist_id = -1;
+    s_playlist = Playlist ();
 
-    for (int p = 0; playlist_id < 0 && p < aud_playlist_count (); p ++)
+    for (int p = 0; p < Playlist::n_playlists (); p ++)
     {
-        String title = aud_playlist_get_title (p);
-        if (! strcmp (title, _("Library")))
-            playlist_id = aud_playlist_get_unique_id (p);
+        auto playlist = Playlist::by_index (p);
+        if (! strcmp (playlist.get_title (), _("Library")))
+        {
+            s_playlist = playlist;
+            break;
+        }
     }
 }
 
-static int create_playlist ()
+static void create_playlist ()
 {
-    int list = aud_playlist_get_blank ();
-    aud_playlist_set_title (list, _("Library"));
-    aud_playlist_set_active (list);
-    playlist_id = aud_playlist_get_unique_id (list);
-    return list;
+    s_playlist = Playlist::blank_playlist ();
+    s_playlist.set_title (_("Library"));
+    s_playlist.active_playlist ();
 }
 
-static int get_playlist (bool require_added, bool require_scanned)
+static bool check_playlist (bool require_added, bool require_scanned)
 {
-    if (playlist_id < 0)
-        return -1;
-
-    int list = aud_playlist_by_unique_id (playlist_id);
-
-    if (list < 0)
+    if (! s_playlist.exists ())
     {
-        playlist_id = -1;
-        return -1;
+        s_playlist = Playlist ();
+        return false;
     }
 
-    if (require_added && aud_playlist_add_in_progress (list))
-        return -1;
-    if (require_scanned && aud_playlist_scan_in_progress (list))
-        return -1;
+    if (require_added && s_playlist.add_in_progress ())
+        return false;
+    if (require_scanned && s_playlist.scan_in_progress ())
+        return false;
 
-    return list;
+    return true;
 }
 
 static String get_uri ()
@@ -245,21 +254,21 @@ static String get_uri ()
 
 static void destroy_database ()
 {
-    items.clear ();
-    hidden_items = 0;
-    database.clear ();
-    database_valid = false;
+    s_items.clear ();
+    s_hidden_items = 0;
+    s_database.clear ();
+    s_database_valid = false;
 }
 
-static void create_database (int list)
+static void create_database ()
 {
     destroy_database ();
 
-    int entries = aud_playlist_entry_count (list);
+    int entries = s_playlist.n_entries ();
 
     for (int e = 0; e < entries; e ++)
     {
-        Tuple tuple = aud_playlist_entry_get_tuple (list, e, Playlist::NoWait);
+        Tuple tuple = s_playlist.entry_tuple (e, Playlist::NoWait);
 
         aud::array<SearchField, String> fields;
         fields[SearchField::Genre] = tuple.get_str (Tuple::Genre);
@@ -268,7 +277,7 @@ static void create_database (int list)
         fields[SearchField::Title] = tuple.get_str (Tuple::Title);
 
         Item * parent = nullptr;
-        SimpleHash<Key, Item> * hash = & database;
+        SimpleHash<Key, Item> * hash = & s_database;
 
         for (auto f : aud::range<SearchField> ())
         {
@@ -292,34 +301,33 @@ static void create_database (int list)
         }
     }
 
-    database_valid = true;
+    s_database_valid = true;
 }
 
-static void search_cb (const Key & key, Item & item, void * _state)
+static void search_recurse (SimpleHash<Key, Item> & domain, int mask, Index<const Item *> & results)
 {
-    SearchState * state = (SearchState *) _state;
-
-    int oldmask = state->mask;
-    int count = search_terms.len ();
-
-    for (int t = 0, bit = 1; t < count; t ++, bit <<= 1)
+    domain.iterate ([mask, & results] (const Key & key, Item & item)
     {
-        if (! (state->mask & bit))
-            continue; /* skip term if it is already found */
+        int count = s_search_terms.len ();
+        int new_mask = mask;
 
-        if (strstr (item.folded, search_terms[t]))
-            state->mask &= ~bit; /* we found it */
-        else if (! item.children.n_items ())
-            break; /* quit early if there are no children to search */
-    }
+        for (int t = 0, bit = 1; t < count; t ++, bit <<= 1)
+        {
+            if (! (new_mask & bit))
+                continue; /* skip term if it is already found */
 
-    /* adding an item with exactly one child is redundant, so avoid it */
-    if (! state->mask && item.children.n_items () != 1)
-        state->items.append (& item);
+            if (strstr (item.folded, s_search_terms[t]))
+                new_mask &= ~bit; /* we found it */
+            else if (! item.children.n_items ())
+                break; /* quit early if there are no children to search */
+        }
 
-    item.children.iterate (search_cb, state);
+        /* adding an item with exactly one child is redundant, so avoid it */
+        if (! new_mask && item.children.n_items () != 1)
+            results.append (& item);
 
-    state->mask = oldmask;
+        search_recurse (item.children, new_mask, results);
+    });
 }
 
 static int item_compare (const Item * const & a, const Item * const & b)
@@ -351,120 +359,112 @@ static int item_compare_pass1 (const Item * const & a, const Item * const & b)
 
 static void do_search ()
 {
-    items.clear ();
-    hidden_items = 0;
+    s_items.clear ();
+    s_hidden_items = 0;
 
-    if (! database_valid)
+    if (! s_database_valid)
         return;
 
-    SearchState state;
-
     /* effectively limits number of search terms to 32 */
-    state.mask = (1 << search_terms.len ()) - 1;
-
-    database.iterate (search_cb, & state);
-
-    items = std::move (state.items);
+    search_recurse (s_database, (1 << s_search_terms.len ()) - 1, s_items);
 
     /* first sort by number of songs per item */
-    items.sort (item_compare_pass1);
+    s_items.sort (item_compare_pass1);
 
     /* limit to items with most songs */
-    if (items.len () > MAX_RESULTS)
+    if (s_items.len () > MAX_RESULTS)
     {
-        hidden_items = items.len () - MAX_RESULTS;
-        items.remove (MAX_RESULTS, -1);
+        s_hidden_items = s_items.len () - MAX_RESULTS;
+        s_items.remove (MAX_RESULTS, -1);
     }
 
     /* sort by item type, then item name */
-    items.sort (item_compare);
+    s_items.sort (item_compare);
 }
 
 static bool filter_cb (const char * filename, void * unused)
 {
     bool add = false;
-    tiny_lock (& adding_lock);
+    tiny_lock (& s_adding_lock);
 
-    if (adding)
+    if (s_adding)
     {
-        bool * added = added_table.lookup (String (filename));
+        bool * added = s_added_table.lookup (String (filename));
 
         if ((add = ! added))
-            added_table.add (String (filename), true);
+            s_added_table.add (String (filename), true);
         else
             (* added) = true;
     }
 
-    tiny_unlock (& adding_lock);
+    tiny_unlock (& s_adding_lock);
     return add;
 }
 
 static void begin_add (const char * uri)
 {
-    if (adding)
+    if (s_adding)
         return;
 
-    int list = get_playlist (false, false);
-
-    if (list < 0)
-        list = create_playlist ();
+    if (! check_playlist (false, false))
+        create_playlist ();
 
     /* if possible, store local path for compatibility with older versions */
     StringBuf path = uri_to_filename (uri);
     aud_set_str ("search-tool", "path", path ? path : uri);
 
-    added_table.clear ();
+    s_added_table.clear ();
 
-    int entries = aud_playlist_entry_count (list);
+    int entries = s_playlist.n_entries ();
 
     for (int entry = 0; entry < entries; entry ++)
     {
-        String filename = aud_playlist_entry_get_filename (list, entry);
+        String filename = s_playlist.entry_filename (entry);
 
-        if (! added_table.lookup (filename))
+        if (! s_added_table.lookup (filename))
         {
-            aud_playlist_entry_set_selected (list, entry, false);
-            added_table.add (filename, false);
+            s_playlist.select_entry (entry, false);
+            s_added_table.add (filename, false);
         }
         else
-            aud_playlist_entry_set_selected (list, entry, true);
+            s_playlist.select_entry (entry, true);
     }
 
-    aud_playlist_delete_selected (list);
+    s_playlist.remove_selected ();
 
-    tiny_lock (& adding_lock);
-    adding = true;
-    tiny_unlock (& adding_lock);
+    tiny_lock (& s_adding_lock);
+    s_adding = true;
+    tiny_unlock (& s_adding_lock);
 
     Index<PlaylistAddItem> add;
     add.append (String (uri));
-    aud_playlist_entry_insert_filtered (list, -1, std::move (add), filter_cb, nullptr, false);
+    s_playlist.insert_filtered (-1, std::move (add), filter_cb, nullptr, false);
 }
 
 static void show_hide_widgets ()
 {
-    if (playlist_id < 0)
+    if (s_playlist == Playlist ())
     {
-        wait_label->hide ();
-        results_list->hide ();
-        stats_label->hide ();
-        help_label->show ();
+        s_wait_label->hide ();
+        s_results_list->hide ();
+        s_stats_label->hide ();
+        s_help_label->show ();
     }
     else
     {
-        help_label->hide ();
+        s_help_label->hide ();
 
-        if (database_valid)
+        if (s_database_valid)
         {
-            wait_label->hide ();
-            results_list->show ();
-            stats_label->show ();
+            s_wait_label->hide ();
+            s_results_list->show ();
+            s_stats_label->show ();
         }
         else
         {
-            results_list->hide ();
-            stats_label->hide ();
-            wait_label->show ();
+            s_results_list->hide ();
+            s_stats_label->hide ();
+            s_wait_label->show ();
         }
     }
 }
@@ -473,106 +473,96 @@ static void search_timeout (void * = nullptr)
 {
     do_search ();
 
-    model.update ();
+    s_model.update ();
 
-    if (items.len ())
+    if (s_items.len ())
     {
-        auto sel = results_list->selectionModel ();
-        sel->select (model.index (0, 0), sel->Clear | sel->SelectCurrent);
+        auto sel = s_results_list->selectionModel ();
+        sel->select (s_model.index (0, 0), sel->Clear | sel->SelectCurrent);
     }
 
-    int total = items.len () + hidden_items;
-    StringBuf stats = str_printf (dngettext (PACKAGE, "%d result",
-     "%d results", total), total);
+    int total = s_items.len () + s_hidden_items;
 
-    if (hidden_items)
-    {
-        stats.insert (-1, " ");
-        stats.combine (str_printf (dngettext (PACKAGE, "(%d hidden)",
-         "(%d hidden)", hidden_items), hidden_items));
-    }
+    if (s_hidden_items)
+        s_stats_label->setText ((const char *)
+         str_printf (dngettext (PACKAGE, "%d of %d result shown",
+         "%d of %d results shown", total), s_items.len (), total));
+    else
+        s_stats_label->setText ((const char *)
+         str_printf (dngettext (PACKAGE, "%d result", "%d results", total), total));
 
-    stats_label->setText ((const char *) stats);
-
-    search_timer.stop ();
-    search_pending = false;
+    s_search_timer.stop ();
+    s_search_pending = false;
 }
 
 static void update_database ()
 {
-    int list = get_playlist (true, true);
-
-    if (list >= 0)
+    if (check_playlist (true, true))
     {
-        create_database (list);
+        create_database ();
         search_timeout ();
     }
     else
     {
         destroy_database ();
-        model.update ();
-        stats_label->clear ();
+        s_model.update ();
+        s_stats_label->clear ();
     }
 
     show_hide_widgets ();
 }
 
-static void add_complete_cb (void * unused, void * unused2)
+static void add_complete_cb (void *, void *)
 {
-    int list = get_playlist (true, false);
-    if (list < 0)
+    if (! check_playlist (true, false))
         return;
 
-    if (adding)
+    if (s_adding)
     {
-        tiny_lock (& adding_lock);
-        adding = false;
-        tiny_unlock (& adding_lock);
+        tiny_lock (& s_adding_lock);
+        s_adding = false;
+        tiny_unlock (& s_adding_lock);
 
-        int entries = aud_playlist_entry_count (list);
+        int entries = s_playlist.n_entries ();
 
         for (int entry = 0; entry < entries; entry ++)
         {
-            String filename = aud_playlist_entry_get_filename (list, entry);
-            bool * added = added_table.lookup (filename);
+            String filename = s_playlist.entry_filename (entry);
+            bool * added = s_added_table.lookup (filename);
 
-            aud_playlist_entry_set_selected (list, entry, ! added || ! (* added));
+            s_playlist.select_entry (entry, ! added || ! (* added));
         }
 
-        added_table.clear ();
+        s_added_table.clear ();
 
         /* don't clear the playlist if nothing was added */
-        if (aud_playlist_selected_count (list) < aud_playlist_entry_count (list))
-            aud_playlist_delete_selected (list);
+        if (s_playlist.n_selected () < entries)
+            s_playlist.remove_selected ();
         else
-            aud_playlist_select_all (list, false);
+            s_playlist.select_all (false);
 
-        aud_playlist_sort_by_scheme (list, Playlist::Path);
+        s_playlist.sort_entries (Playlist::Path);
     }
 
-    if (! database_valid && ! aud_playlist_update_pending (list))
+    if (! s_database_valid && ! s_playlist.update_pending ())
         update_database ();
 }
 
-static void scan_complete_cb (void * unused, void * unused2)
+static void scan_complete_cb (void *, void *)
 {
-    int list = get_playlist (true, true);
-    if (list < 0)
+    if (! check_playlist (true, true))
         return;
 
-    if (! database_valid && ! aud_playlist_update_pending (list))
+    if (! s_database_valid && ! s_playlist.update_pending ())
         update_database ();
 }
 
-static void playlist_update_cb (void * data, void * unused)
+static void playlist_update_cb (void *, void *)
 {
-    if (! database_valid)
-        update_database ();
-    else
+    if (! s_database_valid || ! check_playlist (true, true) ||
+        s_playlist.update_detail ().level >= Playlist::Metadata)
     {
-        int list = get_playlist (true, true);
-        if (list < 0 || aud_playlist_update_detail (list).level >= Playlist::Metadata)
-            update_database ();
+        update_database ();
     }
 }
 
@@ -593,49 +583,52 @@ static void search_cleanup ()
     hook_dissociate ("playlist scan complete", scan_complete_cb);
     hook_dissociate ("playlist update", playlist_update_cb);
 
-    search_timer.stop ();
-    search_pending = false;
+    s_search_timer.stop ();
+    s_search_pending = false;
 
-    search_terms.clear ();
-    items.clear ();
+    s_search_terms.clear ();
+    s_items.clear ();
 
-    tiny_lock (& adding_lock);
-    adding = false;
-    tiny_unlock (& adding_lock);
+    tiny_lock (& s_adding_lock);
+    s_adding = false;
+    tiny_unlock (& s_adding_lock);
 
-    added_table.clear ();
+    s_added_table.clear ();
     destroy_database ();
 
-    delete menu;
-    menu = nullptr;
+    s_help_label = s_wait_label = s_stats_label = nullptr;
+    s_search_entry = nullptr;
+    s_results_list = nullptr;
+
+    delete s_menu;
+    s_menu = nullptr;
 }
 
 static void do_add (bool play, bool set_title)
 {
-    if (search_pending)
+    if (s_search_pending)
         search_timeout ();
 
-    int list = aud_playlist_by_unique_id (playlist_id);
-    int n_items = items.len ();
+    int n_items = s_items.len ();
     int n_selected = 0;
 
     Index<PlaylistAddItem> add;
     String title;
 
-    for (auto & idx : results_list->selectionModel ()->selectedRows ())
+    for (auto & idx : s_results_list->selectionModel ()->selectedRows ())
     {
         int i = idx.row ();
         if (i < 0 || i >= n_items)
             continue;
 
-        const Item * item = items[i];
+        const Item * item = s_items[i];
 
         for (int entry : item->matches)
         {
             add.append (
-                aud_playlist_entry_get_filename (list, entry),
-                aud_playlist_entry_get_tuple (list, entry, Playlist::NoWait),
-                aud_playlist_entry_get_decoder (list, entry, Playlist::NoWait)
+                s_playlist.entry_filename (entry),
+                s_playlist.entry_tuple (entry, Playlist::NoWait),
+                s_playlist.entry_decoder (entry, Playlist::NoWait)
             );
         }
 
@@ -644,37 +637,37 @@ static void do_add (bool play, bool set_title)
             title = item->name;
     }
 
-    int list2 = aud_playlist_get_active ();
-    aud_playlist_entry_insert_batch (list2, -1, std::move (add), play);
+    auto list2 = Playlist::active_playlist ();
+    list2.insert_items (-1, std::move (add), play);
 
     if (set_title && n_selected == 1)
-        aud_playlist_set_title (list2, title);
+        list2.set_title (title);
 }
 
 static void action_play ()
 {
-    aud_playlist_set_active (aud_playlist_get_temporary ());
+    Playlist::temporary_playlist ().activate ();
     do_add (true, false);
 }
 
 static void action_create_playlist ()
 {
-    aud_playlist_new ();
+    Playlist::new_playlist ();
     do_add (false, true);
 }
 
 static void action_add_to_playlist ()
 {
-    if (aud_playlist_by_unique_id (playlist_id) != aud_playlist_get_active ())
+    if (s_playlist != Playlist::active_playlist ())
         do_add (false, false);
 }
 
 static StringBuf create_item_label (int row)
 {
-    if (row < 0 || row >= items.len ())
+    if (row < 0 || row >= s_items.len ())
         return StringBuf ();
 
-    const Item * item = items[row];
+    const Item * item = s_items[row];
     StringBuf string = str_concat ({item->name, "\n"});
 
     if (item->field != SearchField::Title)
@@ -690,12 +683,14 @@ static StringBuf create_item_label (int row)
         string.insert (-1, _("of this genre"));
     }
 
-    while ((item = item->parent))
+    if (item->parent)
     {
+        auto parent = (item->parent->parent ? item->parent->parent : item->parent);
+
         string.insert (-1, " ");
-        string.insert (-1, (item->field == SearchField::Album) ? _("on") : _("by"));
+        string.insert (-1, (parent->field == SearchField::Album) ? _("on") : _("by"));
         string.insert (-1, " ");
-        string.insert (-1, item->name);
+        string.insert (-1, parent->name);
     }
 
     return string;
@@ -709,68 +704,106 @@ void ResultsView::contextMenuEvent (QContextMenuEvent * event)
         audqt::MenuCommand ({N_("_Add to Playlist"), "list-add"}, action_add_to_playlist)
     };
 
-    if (! menu)
-        menu = audqt::menu_build ({items});
+    if (! s_menu)
+        s_menu = audqt::menu_build ({items});
 
-    menu->popup (event->globalPos ());
+    s_menu->popup (event->globalPos ());
+}
+
+QMimeData * ResultsModel::mimeData (const QModelIndexList & indexes) const
+{
+    if (s_search_pending)
+        search_timeout ();
+
+    s_playlist.select_all (false);
+
+    QList<QUrl> urls;
+    for (auto & index : indexes)
+    {
+        int row = index.row ();
+        if (row < 0 || row >= s_items.len ())
+            continue;
+
+        for (int entry : s_items[row]->matches)
+        {
+            urls.append (QString (s_playlist.entry_filename (entry)));
+            s_playlist.select_entry (entry, true);
+        }
+    }
+
+    s_playlist.cache_selected ();
+
+    auto data = new QMimeData;
+    data->setUrls (urls);
+    return data;
 }
 
 void * SearchToolQt::get_qt_widget ()
 {
-    auto widget = new QWidget;
-    auto vbox = new QVBoxLayout (widget);
-    vbox->setContentsMargins (0, 0, 0, 0);
+    s_search_entry = new QLineEdit;
+    s_search_entry->setContentsMargins (audqt::margins.TwoPt);
+    s_search_entry->setClearButtonEnabled (true);
+    s_search_entry->setPlaceholderText (_("Search library"));
 
-    auto entry = new QLineEdit;
-    entry->setPlaceholderText (_("Search library"));
-    vbox->addWidget (entry);
-
-    help_label = new QLabel (_("To import your music library into Audacious, "
+    s_help_label = new QLabel (_("To import your music library into Audacious, "
      "choose a folder and then click the \"refresh\" icon."));
-    help_label->setAlignment (Qt::AlignCenter);
-    help_label->setWordWrap (true);
-    vbox->addWidget (help_label);
+    s_help_label->setAlignment (Qt::AlignCenter);
+    s_help_label->setContentsMargins (audqt::margins.EightPt);
+    s_help_label->setWordWrap (true);
 
-    wait_label = new QLabel (_("Please wait ..."));
-    wait_label->setAlignment (Qt::AlignCenter);
-    vbox->addWidget (wait_label);
+    s_wait_label = new QLabel (_("Please wait ..."));
+    s_wait_label->setAlignment (Qt::AlignCenter);
+    s_wait_label->setContentsMargins (audqt::margins.EightPt);
 
-    results_list = new ResultsView;
-    results_list->setHeaderHidden (true);
-    results_list->setIndentation (0);
-    results_list->setModel (& model);
-    results_list->setSelectionMode (QTreeView::ExtendedSelection);
-    vbox->addWidget (results_list);
+    s_results_list = new ResultsView;
+    s_results_list->setFrameStyle (QFrame::NoFrame);
+    s_results_list->setHeaderHidden (true);
+    s_results_list->setIndentation (0);
+    s_results_list->setModel (& s_model);
+    s_results_list->setSelectionMode (QTreeView::ExtendedSelection);
+    s_results_list->setDragDropMode (QTreeView::DragOnly);
 
-    stats_label = new QLabel;
-    stats_label->setAlignment (Qt::AlignCenter);
-    vbox->addWidget (stats_label);
-
-    auto hbox = new QHBoxLayout;
-    vbox->addLayout (hbox);
+    s_stats_label = new QLabel;
+    s_stats_label->setAlignment (Qt::AlignCenter);
+    s_stats_label->setContentsMargins (audqt::margins.TwoPt);
 
     auto chooser = new QLineEdit;
-    hbox->addWidget (chooser);
 
     auto button = new QPushButton (QIcon::fromTheme ("view-refresh"), QString ());
     button->setFlat (true);
+    button->setFocusPolicy (Qt::NoFocus);
+
+    auto hbox = audqt::make_hbox (nullptr);
+    hbox->setContentsMargins (audqt::margins.TwoPt);
+
+    hbox->addWidget (chooser);
     hbox->addWidget (button);
 
+    auto widget = new QWidget;
+    auto vbox = audqt::make_vbox (widget, 0);
+
+    vbox->addWidget (s_search_entry);
+    vbox->addWidget (s_help_label);
+    vbox->addWidget (s_wait_label);
+    vbox->addWidget (s_results_list);
+    vbox->addWidget (s_stats_label);
+    vbox->addLayout (hbox);
+
     String uri = get_uri ();
     StringBuf path = uri_to_filename (uri, false);
     chooser->setText (path ? path : uri);
 
     search_init ();
 
-    QObject::connect (vbox, & QObject::destroyed, search_cleanup);
-    QObject::connect (entry, & QLineEdit::returnPressed, action_play);
-    QObject::connect (results_list, & QTreeView::doubleClicked, action_play);
+    QObject::connect (widget, & QObject::destroyed, search_cleanup);
+    QObject::connect (s_search_entry, & QLineEdit::returnPressed, action_play);
+    QObject::connect (s_results_list, & QTreeView::doubleClicked, action_play);
 
-    QObject::connect (entry, & QLineEdit::textEdited, [] (const QString & text)
+    QObject::connect (s_search_entry, & QLineEdit::textEdited, [] (const QString & text)
     {
-        search_terms = str_list_to_index (str_tolower_utf8 (text.toUtf8 ()), " ");
-        search_timer.queue (SEARCH_DELAY, search_timeout, nullptr);
-        search_pending = true;
+        s_search_terms = str_list_to_index (str_tolower_utf8 (text.toUtf8 ()), " ");
+        s_search_timer.queue (SEARCH_DELAY, search_timeout, nullptr);
+        s_search_pending = true;
     });
 
     QObject::connect (chooser, & QLineEdit::textEdited, [button] (const QString & text)
@@ -790,3 +823,14 @@ void * SearchToolQt::get_qt_widget ()
 
     return widget;
 }
+
+int SearchToolQt::take_message (const char * code, const void *, int)
+{
+    if (! strcmp (code, "grab focus") && s_search_entry)
+    {
+        s_search_entry->setFocus (Qt::OtherFocusReason);
+        return 0;
+    }
+
+    return -1;
+}
