@@ -37,6 +37,8 @@
 #include <libaudcore/runtime.h>
 #include <libaudqt/libaudqt.h>
 
+#include "../ui-common/dialogs-qt.h"
+
 #include "actions-mainwin.h"
 #include "actions-playlist.h"
 #include "dnd.h"
@@ -69,9 +71,12 @@ public:
     MainWindow (bool shaded) :
         Window (WINDOW_MAIN, & config.player_x, & config.player_y,
          shaded ? MAINWIN_SHADED_WIDTH : skin.hints.mainwin_width,
-         shaded ? MAINWIN_SHADED_HEIGHT : skin.hints.mainwin_height, shaded) {}
+         shaded ? MAINWIN_SHADED_HEIGHT : skin.hints.mainwin_height, shaded),
+        m_dialogs (this) {}
 
 private:
+    DialogWindows m_dialogs;
+
     void draw (QPainter & cr);
     bool button_press (QMouseEvent * event);
     bool scroll (QWheelEvent * event);
@@ -129,6 +134,7 @@ static void format_time (char buf[7], int time, int length)
     if (remaining && length > 0)
     {
         time = (length - time) / 1000;
+        time = aud::clamp(0, time, 359999); // 99:59:59
 
         if (time < 60)
             snprintf (buf, 7, zero ? "-00:%02d" : " -0:%02d", time);
@@ -140,6 +146,7 @@ static void format_time (char buf[7], int time, int length)
     else
     {
         time /= 1000;
+        time = aud::clamp(0, time, 3599999); // 999:59:59
 
         if (time < 6000)
             snprintf (buf, 7, zero ? " %02d:%02d" : " %2d:%02d", time / 60, time % 60);
@@ -208,11 +215,18 @@ void mainwin_show_status_message (const char * message)
 
 static void mainwin_set_song_title (const char * title)
 {
-    if (title)
-        mainwin->setWindowTitle ((const char *) str_printf (_("%s - Audacious"), title));
-    else
-        mainwin->setWindowTitle (_("Audacious"));
+    StringBuf buf;
 
+    if (title)
+        buf.steal (str_printf (_("%s - Audacious"), title));
+    else
+        buf.steal (str_copy (_("Audacious")));
+
+    int instance = aud_get_instance ();
+    if (instance != 1)
+        buf.combine (str_printf (" (%d)", instance));
+
+    mainwin->setWindowTitle ((const char *) buf);
     mainwin_set_info_text (title ? title : "");
 }
 
@@ -1056,7 +1070,6 @@ static void mainwin_create_window ()
     bool shaded = aud_get_bool ("skins", "player_shaded");
 
     mainwin = new MainWindow (shaded);
-    mainwin->setWindowTitle (_("Audacious"));
 
 #if 0
     GtkWidget * w = mainwin->gtk ();
@@ -1111,6 +1124,7 @@ void mainwin_create ()
 {
     mainwin_create_window ();
     mainwin_create_widgets ();
+    mainwin_set_song_title (nullptr);
 }
 
 static void mainwin_update_volume ()
