@@ -47,7 +47,8 @@ public:
         N_("Status Icon"),
         PACKAGE,
         about,
-        & prefs
+        & prefs,
+        PluginGLibOnly
     };
 
     constexpr StatusIcon () : GeneralPlugin (info, false) {}
@@ -72,7 +73,7 @@ static void si_menu_show (int x, int y, unsigned button, uint32_t time, void *);
 static void si_popup_hide (void * icon);
 
 const char * const StatusIcon::defaults[] = {
-    "scroll_action", "0", /* SI_CFG_SCROLL_ACTION_VOLUME */
+    "scroll_action", aud::numeric_string<SI_CFG_SCROLL_ACTION_VOLUME>::str,
     "volume_delta", "5",
     "disable_popup", "FALSE",
     "close_to_tray", "FALSE",
@@ -273,13 +274,13 @@ static void open_files ()
 static GtkWidget * si_menu_create ()
 {
     static const AudguiMenuItem items[] = {
-        MenuCommand (N_("_Open Files ..."), "document-open", 0, (GdkModifierType) 0, open_files),
-        MenuCommand (N_("Pre_vious"), "media-skip-backward", 0, (GdkModifierType) 0, aud_drct_pl_prev),
         MenuCommand (N_("_Play"), "media-playback-start", 0, (GdkModifierType) 0, aud_drct_play),
         MenuCommand (N_("Paus_e"), "media-playback-pause", 0, (GdkModifierType) 0, aud_drct_pause),
         MenuCommand (N_("_Stop"), "media-playback-stop", 0, (GdkModifierType) 0, aud_drct_stop),
+        MenuCommand (N_("Pre_vious"), "media-skip-backward", 0, (GdkModifierType) 0, aud_drct_pl_prev),
         MenuCommand (N_("_Next"), "media-skip-forward", 0, (GdkModifierType) 0, aud_drct_pl_next),
         MenuSep (),
+        MenuCommand (N_("_Open Files ..."), "document-open", 0, (GdkModifierType) 0, open_files),
         MenuCommand (N_("Se_ttings ..."), "preferences-system", 0, (GdkModifierType) 0, audgui_show_prefs_window),
         MenuCommand (N_("_Quit"), "application-exit", 0, (GdkModifierType) 0, aud_quit)
     };
@@ -353,12 +354,13 @@ static void si_enable (bool enable)
 
 bool StatusIcon::init ()
 {
-    if (aud_get_mainloop_type () != MainloopType::GLib)
-        return false;
-
     aud_config_set_defaults ("statusicon", defaults);
     audgui_init ();
     si_enable (true);
+
+    if (! aud_ui_is_shown ())
+        gdk_notify_startup_complete ();
+
     return true;
 }
 
